<?php

declare(strict_types=1);

namespace Drupal\violinist_teams\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use Drupal\violinist_teams\TeamNode;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Violinist teams form.
 */
final class TeamSettingsForm extends FormBase {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected AccountProxyInterface $currentUser;

  /**
   * Team node.
   *
   * @var \Drupal\violinist_teams\TeamNode
   */
  protected TeamNode $team;

  /**
   * The constructor.
   */
  public function __construct(CurrentRouteMatch $current_route_match, AccountProxyInterface $current_user) {
    $this->routeMatch = $current_route_match;
    $this->currentUser = $current_user;
    $team = $this->routeMatch->getParameter('team');
    if ($team instanceof TeamNode) {
      $this->team = $team;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_route_match'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'violinist_teams_team_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $team = $this->team;
    if (!$team instanceof TeamNode) {
      return $form;
    }
    $form['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Team Name'),
      '#required' => TRUE,
      '#default_value' => $team->getTitle(),
    ];
    if (!$team->isAdmin($this->currentUser)) {
      $form['title']['#description'] = $this->t('Only administrators can change the team name.');
      $form['title']['#disabled'] = TRUE;
      $form['title']['#attributes']['readonly'] = 'readonly';
      $form['title']['#attributes']['disabled'] = 'disabled';
    }
    $form['slack'] = [
      '#type' => 'details',
      '#title' => $this->t('Slack settings'),
      '#open' => TRUE,
    ];
    $form['slack']['enable_slack_notifications'] = [
      '#type' => 'checkbox',
      '#default_value' => $team->isNotificationsEnabledForSlack(),
      '#title' => $this->t('Enable Slack notifications'),
    ];
    if (!$team->getTeamSlackWebhook()) {
      $form['slack']['enable_slack_notifications']['#disabled'] = TRUE;
      $form['slack']['enable_slack_notifications']['#description'] = $this->getSlackDescription();
    }
    $form['slack'] = [
      '#type' => 'details',
      '#title' => $this->t('Slack settings'),
      '#open' => TRUE,
    ];
    $form['slack']['enable_slack_notifications'] = [
      '#type' => 'checkbox',
      '#default_value' => $team->isNotificationsEnabledForSlack(),
      '#title' => $this->t('Enable Slack notifications'),
    ];
    if (!$team->getTeamSlackWebhook()) {
      $form['slack']['enable_slack_notifications']['#disabled'] = TRUE;
      $form['slack']['enable_slack_notifications']['#description'] = $this->getSlackDescription();
    }
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * Helper to get that description.
   */
  protected function getSlackDescription() {
    return t('You must <a href="@url" class="underline">connect your user to slack</a> to enable this feature', [
      '@url' => Url::fromRoute('league_oauth_login.login_controller_link', [
        'provider_id' => 'slack',
      ], [
        'query' => [
          'team' => $this->team?->id() ?? NULL,
        ],
      ])->toString(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $team = $this->team;
    if (!$team instanceof TeamNode) {
      return;
    }
    if ($this->team->isAdmin($this->currentUser)) {
      $team->setTitle($form_state->getValue('title'));
    }
    $team
      ->set(TeamNode::SLACK_NOTIFICATION_ENABLED_FIELD, $form_state->getValue('enable_slack_notifications'))
      ->save();
  }

}
